/*desiredModes
 * modes / 0 / zoneID
 * modes / 0 / EcoTemperatur
 * modes / 0 / EcoActive
 * modes / 0 / NightTemperatur
 * modes / 0 / NightActive
 * modes / 0 / HolidayTemperatur
 * modes / 0 / HolidayActive
 * modes / 0 / ComfortTemperatur
 * modes / 0 / ComfortActive
 * modes / 0 / ManualTemperatur
 * modes / 0 / ManualActive
 * 
 * modes / x / zoneID
 * modes / x / EcoTemperatur
 * modes / x / EcoActive
 * modes / x / NightTemperatur
 * modes / x / NightActive
 * modes / x / HolidayTemperatur
 * modes / x / HolidayActive
 * modes / x / ComfortTemperatur
 * modes / x / ComfortActive
 * modes / 0 / ManualTemperatur
 * modes / 0 / ManualActive
 * modes / x / notUsedTemperatur
 * modes / x / notUsedActive
 * */


var OperationModes = {
	Off : 0,
	Comfort : 1,
	Eco : 2,
	NotUsed : 3,
	Night : 4,
	Holiday : 5,
	Manual : 6,
	nothing : -1
};


var stateValues = {
	active:1,
	inactive:2
};

var fInSync = false;

function reactOnPresent() {
	if (Property.getProperty('generalSettings/useNotUsedOnAbsent') == null) {
		lInfo.logln('generalSettings/useNotUsedOnAbsent is null');
		return;
	}

    //reset OpeationModeNotUsed in any case to ensure no deadlock situation if user change settings during absence
	new Event('heating-controller.operation-mode', {
		'actions' : 'resetOperationMode',
		'zoneID' : 0,
		'operationMode' : OperationModes.NotUsed
	}).raise();
    lInfo.logln('reactOnPresent: send event ReturnTime to cloud to delete return time');
	new Event('AddonToCloud',  {"EventName":"ReturnTime","returnOn":""}).raise();
}

function reactOnAbsent() {
	if (Property.getProperty('generalSettings/useNotUsedOnAbsent') == null) {
		lInfo.logln('generalSettings/useNotUsedOnAbsent is null');
		return;
	}

	if (Property.getProperty('generalSettings/useNotUsedOnAbsent') == false) {
		lInfo.logln('generalSettings/useNotUsedOnAbsent is false');
		return;
	}

	new Event('heating-controller.operation-mode', {
		'actions' : 'setOperationMode',
		'zoneID' : 0,
		'operationMode' : OperationModes.NotUsed
	}).raise();
	
	/*var oArray = Property.getNode('modes').getChildren();
	for ( var iIndex = 0; iIndex < oArray.length; iIndex++) {
		var iZoneID = oArray[iIndex].getChild('zoneID').getValue();
		if (iZoneID != 0) {
			new Event('heating-controller.operation-mode', {
				'actions' : 'setOperationMode',
				'zoneID' : iZoneID,
				'operationMode' : OperationModes.NotUsed,
				'time' : '+' + iIndex
			}).raise();
		}
	}*/
}

function evaluateRealActiveMode(fShouldMessage) {
	var oRoomArray = [];
	var oArray = Property.getNode('modes').getChildren();

	var fApartmentComfort = (getState('operationmode.0.Comfort').getValue()==stateValues.active);
	var fApartmentNight = (getState('operationmode.0.Night').getValue()==stateValues.active);
	var fApartmentHoliday = (getState('operationmode.0.Holiday').getValue()==stateValues.active);
	var fApartmentNotUsed = (getState('operationmode.0.NotUsed').getValue()==stateValues.active);
	
	var zoneID = 0;
	if (raisedEvent.parameter.zoneID != null)
		zoneID = (parseInt("" + raisedEvent.parameter.zoneID, 10));
	lInfo.logln('evaluateRealActiveMode for ' + zoneID);
	var oResArray = [];
	for ( var iIndex = 0; iIndex < oArray.length; iIndex++) {
		try {
			var iZoneID = oArray[iIndex].getChild('zoneID').getValue();

			if (iZoneID)
				if ((zoneID == 0) || (zoneID == iZoneID)) {
					lInfo.logln('check ' + iZoneID);
					var zZone = getZoneByID(parseInt('' + iZoneID, 10));
					var oStatus = zZone.getTemperatureControlStatus();
					var fSkip = false;
					if (oStatus.IsConfigured) {
						lInfo.logln('not configured -> no action');
						fSkip = true;
					}
					if (!fSkip)
						if (oStatus.ControlMode == 0) {
							lInfo.logln('controller off -> no action');
							fSkip = true;
						}
					if (!fSkip)
						if (oStatus.ControlMode == 2) {
							lInfo.logln('controller zone follower -> no action');
							fSkip = true;
						}

					if (!fSkip) {
						var minTemperatur = 100;
						var effectiveMode;
						if (getState('operationmode.'+iZoneID+'.Manual').getValue()==stateValues.active){
							effectiveMode = OperationModes.Manual;
							lInfo.logln('Manuell Mode is allway high prio ' + iZoneID);
						} else if (getState('operationmode.'+iZoneID+'.Holiday').getValue()==stateValues.active) {
							effectiveMode = OperationModes.Holiday;
							lInfo.logln('Holiday Mode is allway high prio ' + iZoneID);
						} else {
							if (fApartmentComfort) {
								minTemperatur = parseFloat("" + oArray[iIndex].getChild('ComfortTemperatur').getValue());
								effectiveMode = OperationModes.Comfort;
							} else {
								minTemperatur = parseFloat("" + oArray[iIndex].getChild('EcoTemperatur').getValue());
								effectiveMode = OperationModes.Eco;
							}
							if (!(zoneID == 0)) {
								if (getState('operationmode.'+iZoneID+'.Comfort').getValue()==stateValues.active) {
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('ComfortTemperatur').getValue());
									effectiveMode = OperationModes.Comfort;
								}
								if (getState('operationmode.'+iZoneID+'.Comfort').getValue()==stateValues.inactive) {
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('EcoTemperatur').getValue());
									effectiveMode = OperationModes.Eco;
								}
							}
							if (getState('operationmode.'+iZoneID+'.Night').getValue()==stateValues.active) {
								if (parseFloat("" + oArray[iIndex].getChild('NightTemperatur').getValue()) < minTemperatur) {
									effectiveMode = OperationModes.Night;
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('NightTemperatur').getValue());
								}
							}
							if ((getState('operationmode.'+iZoneID+'.NotUsed').getValue()==stateValues.active)||fApartmentNotUsed) {
								if (parseFloat("" + oArray[iIndex].getChild('notUsedTemperatur').getValue()) < minTemperatur) {
									effectiveMode = OperationModes.NotUsed;
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('notUsedTemperatur').getValue());
								}
							}
							/*if (oArray[iIndex].getChild('HolidayActive').getValue() || fApartmentHoliday) {
								if (parseFloat("" + oArray[iIndex].getChild('HolidayTemperatur').getValue()) < minTemperatur) {
									effectiveMode = OperationModes.Holiday;
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('HolidayTemperatur').getValue());
								}
							}*/
						}
						//Property.setProperty('modes/' + iZoneID + '/activeMode', effectiveMode);

						if (oStatus.OperationMode != null)
							if (oStatus.OperationMode == effectiveMode)
								fSkip = true;

						if (!fSkip) {
							lInfo.logln('Setting/queueing Mode ' + effectiveMode + ' for zone ' + iZoneID);
							oResArray.push({
								zone : iZoneID,
								value : effectiveMode
							});
						} else {
							lInfo.logln('Setting Mode ' + effectiveMode + ' for zone ' + iZoneID + ' but skipping');
						}
					}
				}
		} catch (e) {
			lWarn.logln('Error on evaluateRealActiveMode for path ' +oArray[iIndex].getPath()+ ' error: ' + e);
		}
	}
	if (oResArray.length > 1) {
		lInfo.logln('#count of action : ' + oResArray.length + ' so queueing ' + JSON.stringify(oResArray));
		
		Property.setProperty('temporaryQueue/actions/0/type','change-addon-state');
   		Property.setProperty('temporaryQueue/actions/0/addon-id','heating-controller');
		Property.setProperty('temporaryQueue/actions/0/statename','queue-active');
		Property.setProperty('temporaryQueue/actions/0/state','active');
		Property.setProperty('temporaryQueue/actions/0/delay',0);

		
		for (var iIndex=0;iIndex<oResArray.length;iIndex++) {
			Property.setProperty('temporaryQueue/actions/' + (iIndex+1) + '/type','zone-scene');
			Property.setProperty('temporaryQueue/actions/' + (iIndex+1) + '/zone',oResArray[iIndex].zone);
			Property.setProperty('temporaryQueue/actions/' + (iIndex+1) + '/group',48);
			Property.setProperty('temporaryQueue/actions/' + (iIndex+1) + '/scene',oResArray[iIndex].value);
			Property.setProperty('temporaryQueue/actions/' + (iIndex+1) + '/delay',0);
			Property.setProperty('temporaryQueue/actions/' + (iIndex+1) + '/category','manual');			
		}
		
		Property.setProperty('temporaryQueue/actions/' + (oResArray.length+1) + '/type','change-addon-state');
   		Property.setProperty('temporaryQueue/actions/' + (oResArray.length+1) + '/addon-id','heating-controller');
		Property.setProperty('temporaryQueue/actions/' + (oResArray.length+1) + '/statename','queue-active');
		Property.setProperty('temporaryQueue/actions/' + (oResArray.length+1) + '/state','inactive');
		Property.setProperty('temporaryQueue/actions/' + (oResArray.length+1) + '/delay',0);
		

		new Event('action_execute', {
			path : '/scripts/heating-controller/temporaryQueue'
		}).raise();
		
		/*new Event('heating-controller.operation-mode', {
			actions : 'queue',
			queue : JSON.stringify(oResArray),
			index : 0,
			fShouldMessage : fShouldMessage
		}).raise();*/
	} else {
		if (oResArray.length == 1) {
			lInfo.logln('#count of action 1 so executing directly');
			var pZone = getZoneByID(oResArray[0].zone);
			pZone.callScene(48, oResArray[0].value);
		}
		lInfo.logln('finished!!!!');
		new Event('heating-controller.answer', {
			'requestingEvent' : 'heating-controller.operation-mode',
			'requestingAction' : 'setOperationMode',
			'result' : 0
		}).raise();
		lInfo.logln('finished!!!!');
	}
}

function setOperationMode() {
	var iZoneID = parseInt("" + raisedEvent.parameter.zoneID, 10);

	var operationMode = -1;
	if (raisedEvent.parameter.operationMode != null)
		operationMode = parseInt("" + raisedEvent.parameter.operationMode, 10);
	if (raisedEvent.parameter.desiredOperationMode != null)
		operationMode = parseInt("" + raisedEvent.parameter.desiredOperationMode, 10);

	
	if (iZoneID == 0) {
		lInfo.logln('set OperationMode for apartment to ' + operationMode);
		switch (operationMode) {
		case OperationModes.Off:
			lInfo.logln('Operationmode Off for Apartment is not specified');
			break;
		case OperationModes.Comfort:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					if (oArray[kIndex].getChild('zoneID')!=null) {
						var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
						lInfo.logln('set OperationMode for '+iZoneID+' to comfort');
						getState('operationmode.' + iZoneID + '.Comfort').setValue(stateValues.active);
						getState('operationmode.' + iZoneID + '.Eco').setValue(stateValues.inactive);
					}
				} catch (e) {
					lWarn.logln("Error on setOperationMode : " + e);
				}
			}
			break;
		case OperationModes.Eco:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					if (oArray[kIndex].getChild('zoneID')!=null) {
						var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
						lInfo.logln('set OperationMode for '+iZoneID+' to eco');
						getState('operationmode.' + iZoneID + '.Comfort').setValue(stateValues.inactive);
						getState('operationmode.' + iZoneID + '.Eco').setValue(stateValues.active);
						lInfo.logln('set OperationMode for '+iZoneID+' done');
					}
				} catch (e) {
					lWarn.logln(e);
				}
			}
			break;
		case OperationModes.NotUsed:			
			getState('operationmode.0.NotUsed').setValue(stateValues.active);
			break;
		case OperationModes.Night:
			// if set night for apartment, all zone must have night also active
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					if (oArray[kIndex].getChild('zoneID')!=null) {
						var iZoneID = oArray[kIndex].getChild('zoneID').getValue();						
						getState('operationmode.' + iZoneID + '.Night').setValue(stateValues.active);
					}
				} catch (e) {}
			}

			break;
		case OperationModes.Holiday:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					if (oArray[kIndex].getChild('zoneID')!=null) {
						var iZoneID = oArray[kIndex].getChild('zoneID').getValue();					
						getState('operationmode.' + iZoneID + '.Holiday').setValue(stateValues.active);
					}
				} catch (e) {}
			}
			break;
		case OperationModes.Manual:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					if (oArray[kIndex].getChild('zoneID')!=null) {
						var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
						getState('operationmode.' + iZoneID + '.Manual').setValue(stateValues.active);
					}
				} catch (e) {}
			}
			break;
		}
	} else {
		switch (operationMode) {
		case OperationModes.Off:
			lInfo.logln('Operationmode Off for Zone is not specified');
			break;
		case OperationModes.Comfort:
			lInfo.logln('Operationmode Comfort for Zone is active');
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(stateValues.inactive);
			getState('operationmode.' + iZoneID + '.Night').setValue(stateValues.inactive);			
			getState('operationmode.' + iZoneID + '.Comfort').setValue(stateValues.active);			
			getState('operationmode.' + iZoneID + '.Eco').setValue(stateValues.inactive);			
			
			break;
		case OperationModes.Eco:
			lInfo.logln('Operationmode ECO for Zone is active');
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(stateValues.inactive);
			getState('operationmode.' + iZoneID + '.Night').setValue(stateValues.inactive);
			getState('operationmode.' + iZoneID + '.Comfort').setValue(stateValues.inactive);
			getState('operationmode.' + iZoneID + '.Eco').setValue(stateValues.active);			
			
			break;
		case OperationModes.NotUsed:
			lInfo.logln('Operationmode Not used for Zone is active');
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(stateValues.active);			
			break;
		case OperationModes.Night:
			getState('operationmode.' + iZoneID + '.Night').setValue(stateValues.active);
			break;
		case OperationModes.Holiday:
			getState('operationmode.' + iZoneID + '.Holiday').setValue(stateValues.active);
			break;
		case OperationModes.Manual:
			getState('operationmode.' + iZoneID + '.Manual').setValue(stateValues.active);
			break;
		}
	}
	Property.store();
}

function resetOperationMode() {
	var iZoneID = parseInt("" + raisedEvent.parameter.zoneID, 10);
	var operationMode = -1;
	if (raisedEvent.parameter.operationMode != null)
		operationMode = parseInt("" + raisedEvent.parameter.operationMode, 10);
	if (raisedEvent.parameter.desiredOperationMode != null)
		operationMode = parseInt("" + raisedEvent.parameter.desiredOperationMode, 10);
	if (iZoneID == 0) {
		switch (operationMode) {
		case OperationModes.Off:
			lInfo.logln('Reseting Operationmode Off for Apartment is not specified');
			break;
		case OperationModes.Comfort:
			lInfo.logln('Reseting Operationmode Comfort for Apartment is not specified');
			break;
		case OperationModes.Eco:
			lInfo.logln('Reseting Operationmode Eco for Apartment is not specified');
			break;
		case OperationModes.NotUsed:
			getState('operationmode.0.NotUsed').setValue(stateValues.inactive);
			break;
		case OperationModes.Night:
			// if night is reset to apartment , also all zones must be reset
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					if (oArray[kIndex].getChild('zoneID')!=null) {
						var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
						getState('operationmode.' + iZoneID + '.Night').setValue(stateValues.inactive);
					}
				} catch (e) {}
			}
			break;
		case OperationModes.Holiday:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					if (oArray[kIndex].getChild('zoneID')!=null) {
						var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
						getState('operationmode.' + iZoneID + '.Holiday').setValue(stateValues.inactive);
					}
				} catch (e) {}
			}
			break;
		case OperationModes.Manual:
			getState('operationmode.0.Manual').setValue(stateValues.inactive);
			break;
		}
	} else {
		switch (operationMode) {
		case OperationModes.Off:
			lInfo.logln('Reseting Operationmode Off for Zone is not specified');
			break;
		case OperationModes.Comfort:
			lInfo.logln('Reseting Operationmode Comfort for Zone is not specified, assume a not used');
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(stateValues.active);
			break;
		case OperationModes.Eco:
			lInfo.logln('Reseting Operationmode Eco for Zone is not specified, assume a not used');
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(stateValues.active);
			break;
		case OperationModes.NotUsed:
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(stateValues.inactive);
			break;
		case OperationModes.Night:
			getState('operationmode.' + iZoneID + '.Night').setValue(stateValues.inactive);
			break;
		case OperationModes.Holiday:
			getState('operationmode.' + iZoneID + '.Holiday').setValue(stateValues.inactive);
			break;
		case OperationModes.Manual:
			getState('operationmode.' + iZoneID + '.Manual').setValue(stateValues.inactive);
			break;
		}
	}
	Property.store();
}

function checkState(ZoneID) {
	if (Property.getNode('/usr/addon-states/heating-controller/operationmode.' + ZoneID + '.Comfort')==null) {
		lInfo.logln('loading states for ' + ZoneID);
		registerState('operationmode.' + ZoneID + '.Comfort',true);
		registerState('operationmode.' + ZoneID + '.Eco',true);
		registerState('operationmode.' + ZoneID + '.Night',true);
		registerState('operationmode.' + ZoneID + '.NotUsed',true);
		registerState('operationmode.' + ZoneID + '.Holiday',true);
		registerState('operationmode.' + ZoneID + '.Manual',true);		
	}
	if ((getState('operationmode.' + ZoneID + '.Comfort').getValue()==stateValues.inactive) &&
		(getState('operationmode.' + ZoneID + '.Eco').getValue()==stateValues.inactive)) {
		lInfo.logln('not existing, inserting ' + ZoneID);
		getState('operationmode.' + ZoneID + '.Comfort').setValue(stateValues.inactive);
		getState('operationmode.' + ZoneID + '.Comfort').setValue(stateValues.active);
		getState('operationmode.' + ZoneID + '.Eco').setValue(stateValues.active);
		getState('operationmode.' + ZoneID + '.Eco').setValue(stateValues.inactive);
		getState('operationmode.' + ZoneID + '.Night').setValue(stateValues.active);
		getState('operationmode.' + ZoneID + '.Night').setValue(stateValues.inactive);
		getState('operationmode.' + ZoneID + '.NotUsed').setValue(stateValues.active);
		getState('operationmode.' + ZoneID + '.NotUsed').setValue(stateValues.inactive);
		getState('operationmode.' + ZoneID + '.Holiday').setValue(stateValues.active);
		getState('operationmode.' + ZoneID + '.Holiday').setValue(stateValues.inactive);
		getState('operationmode.' + ZoneID + '.Manual').setValue(stateValues.active);
		getState('operationmode.' + ZoneID + '.Manual').setValue(stateValues.inactive);
	}
	

	
}

var fInSyncTemperature=false;
function fetchTemperature() {
	if (fInSyncTemperature)
		return;
	fInSyncTemperature = true;
	fetchValuesToLocal();
	
	var oZones = Property.getNode('/apartment/zones').getChildren();

	var zoneID = 0;
	if (raisedEvent.parameter.zoneID != null)
		zoneID = (parseInt("" + raisedEvent.parameter.zoneID, 10));

	for ( var iIndex = 0; iIndex < oZones.length; iIndex++) {
		var iZone = oZones[iIndex].getChild('ZoneID').getValue();
		if (iZone != 0)
			if ((zoneID == 0) || (zoneID == iZone)) {
				try {
					lInfo.logln('fetch temperature value for ' + iZone);
					var zZone = getZoneByID(parseInt('' + iZone, 10));

					var temperatures = zZone.getTemperatureControlValues();
					/*
					 * lInfo.logln('fetch : ' + temperatures); for (var sKey in
					 * temperatures) { lInfo.logln(sKey + ' -> ' +
					 * temperatures[sKey]); }
					 */
					if (temperatures.IsConfigured) {
						checkState(iZone);
						Property.setProperty('modes/' + iZone + '/OffTemperatur', "" + temperatures.Off);
						Property.setProperty('modes/' + iZone + '/EcoTemperatur', "" + temperatures.Economy);
						Property.setProperty('modes/' + iZone + '/NightTemperatur', "" + temperatures.Night);
						Property.setProperty('modes/' + iZone + '/HolidayTemperatur', "" + temperatures.Holiday);
						Property.setProperty('modes/' + iZone + '/ComfortTemperatur', "" + temperatures.Comfort);
						Property.setProperty('modes/' + iZone + '/notUsedTemperatur', "" + temperatures.NotUsed);
					}
				} catch (e) {
					lWarn.logln('Error while fetchTemperature for zone ' + iZone + ' : ' + e);
				}
			}
	}
	
}

function fetchValuesToLocal() {
	if (fInSync)
		return;
	fInSync = true;
	checkState(0);
	Property.setProperty('modes/0/zoneID', 0);

	var oZones = Property.getNode('/apartment/zones').getChildren();

	var zoneID = 0;
	if (raisedEvent.parameter.zoneID != null)
		zoneID = (parseInt("" + raisedEvent.parameter.zoneID, 10));

	for ( var iIndex = 0; iIndex < oZones.length; iIndex++) {
		var iZone = oZones[iIndex].getChild('ZoneID').getValue();
		if (iZone != 0)
			if ((zoneID == 0) || (zoneID == iZone)) {
				try {
					lInfo.logln('fetch state value for ' + iZone);
					var zZone = getZoneByID(parseInt('' + iZone, 10));

					Property.setProperty('modes/' + iZone + '/zoneID', iZone);
					
					checkState(iZone);

				} catch (e) {
					lWarn.logln('Error while fetchValuesToLocal for zone ' + iZone + ' : ' + e);
				}
			}
	}
}

if (raisedEvent.name == 'heating-controller.operation-mode') {
	lInfo.logln('!!!! heating-controller.operation-mode');
	for ( var sKey in raisedEvent.parameter) {
		lInfo.logln(' -> ' + sKey + ' : ' + raisedEvent.parameter[sKey]);
	}
	if (raisedEvent.parameter.actions == 'sync') {
		fetchValuesToLocal();
	}
	if (raisedEvent.parameter.actions == 'evaluateRealActiveMode') {
		fetchTemperature();
		evaluateRealActiveMode(true);
	}
	if (raisedEvent.parameter.actions == "setOperationMode") {
		fetchValuesToLocal();
		setOperationMode();		
		var pParameter={'actions':'evaluateRealActiveMode'};
		if (raisedEvent.parameter.zoneID != null)
			pParameter.zoneID = raisedEvent.parameter.zoneID;
		new Event('heating-controller.operation-mode',pParameter).raise();
	}
	if (raisedEvent.parameter.actions == "resetOperationMode") {
		fetchValuesToLocal();
		resetOperationMode();
		var pParameter={'actions':'evaluateRealActiveMode'};
		if (raisedEvent.parameter.zoneID != null)
			pParameter.zoneID = raisedEvent.parameter.zoneID;
		new Event('heating-controller.operation-mode',pParameter).raise();
	}
	if (raisedEvent.parameter.actions == "queue") {
		lInfo.logln('heating-controller.operation-mode queue ' + unescape(raisedEvent.parameter.queue) + " for index " + raisedEvent.parameter.index);
		var aParameter = JSON.parse(unescape(raisedEvent.parameter.queue));
		var iIndex = parseInt("" + raisedEvent.parameter.index, 10);
		if (iIndex < aParameter.length) {
			var oParam = aParameter[iIndex];
			var zone = oParam.zone;
			var value = oParam.value;
			try {
				lInfo.logln('heating-controller.operation-mode.queue zid ' + zone + ' value ' + value);
				var pZone = getZoneByID(zone);
				pZone.callScene(48, value);
			} catch (e) {
				lWarn.logln(e);
			}
			new Event('heating-controller.operation-mode', {
				actions : 'queue',
				queue : raisedEvent.parameter.queue,
				index : (iIndex + 1),
				time : '+1',
				fShouldMessage : raisedEvent.parameter.fShouldMessage
			}).raise();
		} else {
			lInfo.logln('heating-controller.operation-mode.queue finished');
				new Event('heating-controller.answer', {
					'requestingEvent' : 'heating-controller.operation-mode',
					'requestingAction' : 'setOperationMode',
					'result' : 0
				}).raise();
		}
	}
	if (raisedEvent.parameter.actions == 'reactOnPresent') {
		lInfo.logln('!!! reactOnPresent');
		reactOnPresent();
	}
	if (raisedEvent.parameter.actions == 'reactOnAbsent') {
		lInfo.logln('!!! reactOnAbsent');
		reactOnAbsent();
	}
	if (raisedEvent.parameter.actions=='queue-ready') {
		lInfo.logln('!!!! queue ready remove queue !!!');
		Property.getNode('/scripts/heating-controller').removeChild('temporaryQueue');
		new Event('heating-controller.answer', {
			'requestingEvent' : 'heating-controller.operation-mode',
			'requestingAction' : 'setOperationMode',
			'result' : 0
		}).raise();
		lInfo.logln('!!!! queue ready signalised !!!');
	}
} else if (raisedEvent.name == 'buttonClick') {
    var deviceDSUID = raisedEvent.source.dsuid;
    var deviceNode = getDevices().byDSID(deviceDSUID).getPropertyNode();
    var activeGroupIDNode = deviceNode.getChild('button/activeGroup');
    var activeGroupID;
    if (activeGroupIDNode !== undefined) {
        activeGroupID = activeGroupIDNode.getValue();
    }

    if (activeGroupID == 48) {
        var click = parseInt("" + raisedEvent.parameter.clickType, 10);
        var button = parseInt("" + raisedEvent.parameter.buttonIndex, 10);

        lInfo.logln("click with " + click + " from button " + button);

        if (((button === 0) && (click === 0)) ||
            ((button === 0) && (click === 7)) ||
            ((button === 2) && (click === 0))) {
            // room used == not-NotUsed
            new Event('heating-controller.operation-mode', {
                'actions' : 'resetOperationMode',
                'zoneID' : raisedEvent.source.zoneID,
                'operationMode' : OperationModes.NotUsed
            }).raise();
        } else if (((button === 0) && (click === 6)) ||
                   ((button === 1) && (click === 0))) {
            // room not used == NotUsed
            new Event('heating-controller.operation-mode', {
                'actions' : 'setOperationMode',
                'zoneID' : raisedEvent.source.zoneID,
                'operationMode' : OperationModes.NotUsed
            }).raise();
        }
    }
}
